#include <sysdep.h>


#define NAME __sigaction

	.text
	.align	ALIGN
	.globl	SYMBOL_NAME(NAME)
__sigaction:
	movl	$ __IBCS_sigaction , %eax
	leal	_sigreturn,%edx
	.byte	0x9a, 0, 0, 0, 0, 7, 0
	jnc	LF(1)
	STORE_NUMBER(%eax,ERRNO)
	mov	$-1,%eax
L(1:)
	ret
	.align ALIGN

_sigreturn:
	addl	$4,%esp
	.byte	0x9a, 0, 0, 0, 0, 15, 0

#ifdef ELF_LIBRARY
.type __sigaction,@function
#if 0
.weak sigaction
sigaction = _sigaction
#endif
#endif
