/* Built-in time information for `C' locale,
   generated Thu Jan  1 00:00:00 1970 by li2c.  */

#include <ansidecl.h>
#include <localeinfo.h>
#include <stddef.h>


CONST struct time_info __time_C =
  {
    { 
(char *) "Sun", (char *) "Mon", (char *) "Tue", (char *) "Wed", (char *) "Thu", (char *) "Fri", (char *) "Sat",  },

    { 
(char *) "Sunday", (char *) "Monday", (char *) "Tuesday", (char *) "Wednesday", (char *) "Thursday", (char *) "Friday", (char *) "Saturday",  },
    { 
(char *) "Jan", (char *) "Feb", (char *) "Mar", (char *) "Apr", (char *) "May", (char *) "Jun", (char *) "Jul", (char *) "Aug", (char *) "Sep", (char *) "Oct", (char *) "Nov", (char *) "Dec",  },
    { 
(char *) "January", (char *) "February", (char *) "March", (char *) "April", (char *) "May", (char *) "June", (char *) "July", (char *) "August", (char *) "September", (char *) "October", (char *) "November", (char *) "December",  },
    { (char *) "AM", (char *) "PM" },
    (char *) "%a %b %d %H:%M:%S %Y", (char *) "%m/%d/%y", (char *) "%H:%M:%S",
    (char *) "GMT", (char *) ""
  };

CONST struct time_info *_time_info = &__time_C;
