#include <string.h>

size_t strlen(const char * s)
{
register size_t __res __asm__("cx");
__asm__("cld\n\t"
	"repnz\n\t"
	"scasb\n\t"
	"notl %0\n\t"
	"decl %0"
	:"=c" (__res):"D" (s),"a" (0),"0" (0xffffffff):"di");
return __res;
}
