/*
 * limits.h     iBCS compatable header
 */

/* Portions indiscriminantly copied from limits.c generated by GCC */
/* An attempt was made to simplify the header file so that it only
   contains the iBCS2 stuff and does not need a bunch of other header files */
/* Copyright (C) 1991, 1992 Free Software Foundation, Inc. */

/* Number of bits in a `char'.	*/
#define	CHAR_BIT	8

/* Minimum and maximum values a `signed char' can hold.  */
#define	SCHAR_MIN	(-128)
#define	SCHAR_MAX	127

/* Maximum value an `unsigned char' can hold.  (Minimum is 0.)  */
#ifdef	__STDC__
#define	UCHAR_MAX	255U
#else
#define	UCHAR_MAX	255
#endif

#define	CHAR_MIN	0
#define	CHAR_MAX	UCHAR_MAX

/* Minimum and maximum values a `signed short int' can hold.  */
#define	SHRT_MIN	(-32768)
#define	SHRT_MAX	32767

/* Maximum value an `unsigned short int' can hold.  (Minimum is 0.)  */
#define	USHRT_MAX	65535

/* Minimum and maximum values a `signed int' can hold.  */
#define	INT_MIN	(- INT_MAX - 1)
#define	INT_MAX	2147483647

/* Maximum value an `unsigned int' can hold.  (Minimum is 0.)  */
#ifdef	__STDC__
#define	UINT_MAX	4294967295U
#else
#define	UINT_MAX	4294967295
#endif

/* Minimum and maximum values a `signed long int' can hold.  */
#define	LONG_MIN	INT_MIN
#define	LONG_MAX	INT_MAX

/* Maximum value an `unsigned long int' can hold.  (Minimum is 0.)  */
#define	ULONG_MAX	UINT_MAX

#define WORD_BIT	32
#define LONG_BIT	32

   /* Number of decimal digits of precision in a double */
#undef DBL_DIG
#define DBL_DIG 15

   /* Maximum double */
#undef DBL_MAX
#define DBL_MAX 1.7976931348623157e+308

   /* Minimum normalised double */
#undef DBL_MIN
#define DBL_MIN 2.2250738585072014e-308

   /* Number of decimal digits of precision in a float */
#undef FLT_DIG
#define FLT_DIG 6

   /* Maximum float */
#undef FLT_MAX
#define FLT_MAX 3.40282347e+38F

   /* Minimum normalised float */
#undef FLT_MIN
#define FLT_MIN 1.17549435e-38F

#define CLOCK_TCK 100
#define ARG_MAX 5120
#define LINK_MAX 1000
#define PATH_MAX 256
#define PIPE_BUF 5120
#define PIPE_MAX 5120
#define PASS_MAX 8
#define CHILD_MAX 25
#define SYS_MNLN 9
#define UID_MAX 60000

#define FCHR_MAX 1048576

#define OPEN_MAX         256	/* # open files a process may have */

#undef TZNAME_MAX
#define TZNAME_MAX	50

#ifndef STREAM_MAX
#define STREAM_MAX	OPEN_MAX
#endif
