	.file "crt0.S"
	.type __interpreter_entry,@function
	.globl __interpreter_entry

	.text
__interpreter_entry:
/*
 *	This is important, and was missing from the new version...
 */
	movl $45,%eax
	movl $0,%ebx
	int  $0x80
	movl %eax,___brk_addr
/*
 *	ok, set up the  ___environ and call _main
 */	
	movl 8(%esp),%eax
	movl %eax,_libc_environ
	movzwl __fpu_control,%eax
	pushl %eax
	call __setfpucw
	addl $4,%esp
	call elfinterp_main  /* Never return - the elf interpreter main should
				do the right thing when it is time to exit */
