/*
 *  This file will generate the directive to the linker to load the
 *  shared library, libc_s. In addition, it will define the entry points
 *  which are used in that library.
 *
 */
	.section	.lib		# REQUIRED SECTION NAME
	.long		6		# Total size of this block
	.long		2		# long word count of header
	.ascii		"/shlib/libc_s"	# Name of the library
	.byte		0		# ... must be nul terminated
	.align		4,0		# ... and aligned to 4 byte boundry
/*
 *  Define the procedure entry points and their corresponding locations
 */
	.section	absolute
	.globl		__doprnt
	.set		__doprnt, 0xa0000000

	.globl		__bufsync
	.set		__bufsync, 0xa000005

	.globl		__cleanup
	.set		__cleanup, 0xa000000f

	.globl		__cerror
	.set		__cerror, 0xa000000a

	.globl		__filbuf
	.set		__filbuf, 0xa0000014

	.globl		__findbuf
	.set		__findbuf, 0xa0000019

	.globl		__findiop
	.set		__findiop, 0xa000001e

	.globl		__flsbuf
	.set		__flsbuf, 0xa0000023

	.globl		__wrtchk
	.set		__wrtchk, 0xa0000028

	.globl		__xflsbuf
	.set		__xflsbuf, 0xa000002d

	.globl		_abs
	.set		_abs, 0xa0000032

	.globl		_access
	.set		_access, 0xa0000037

	.globl		_atof
	.set		_atof, 0xa000003c

	.globl		_atoi
	.set		_atoi, 0xa0000041

	.globl		_atol
	.set		_atol, 0xa0000046

	.globl		___brk
	.set		___brk, 0xa000004b

	.globl		_calloc
	.set		_calloc, 0xa0000050

	.globl		_cfree
	.set		_cfree, 0xa0000055

	.globl		_chdir
	.set		_chdir, 0xa000005a

	.globl		_chmod
	.set		_chmod, 0xa000005f

	.globl		_close
	.set		_close, 0xa0000064

	.globl		_creat
	.set		_creat, 0xa0000069

	.globl		_ecvt
	.set		_ecvt, 0xa000006e

	.globl		_fclose
	.set		_fclose, 0xa0000073

	.globl		_fcntl
	.set		_fcntl, 0xa0000078

	.globl		_fcvt
	.set		_fcvt, 0xa000007d

	.globl		_fflush
	.set		_fflush, 0xa0000082

	.globl		_fgetc
	.set		_fgetc, 0xa0000087

	.globl		_fgets
	.set		_fgets, 0xa000008c

	.globl		_fopen
	.set		_fopen, 0xa0000091

	.globl		_fprintf
	.set		_fprintf, 0xa0000096

	.globl		_fputc
	.set		_fputc, 0xa000009b

	.globl		_fputs
	.set		_fputs, 0xa00000a0

	.globl		_fread
	.set		_fread, 0xa00000a5

	.globl		_free
	.set		_free, 0xa00000aa

	.globl		_freopen
	.set		_freopen, 0xa00000af

	.globl		_frexp
	.set		_frexp, 0xa00000b4

	.globl		_fseek
	.set		_fseek, 0xa00000b9

	.globl		_fstat
	.set		_fstat, 0xa00000be

	.globl		_fwrite
	.set		_fwrite, 0xa00000c3

	.globl		_gcvt
	.set		_gcvt, 0xa00000c8

	.globl		_getchar
	.set		_getchar, 0xa00000cd

	.globl		_getenv
	.set		_getenv, 0xa00000d2

	.globl		_getopt
	.set		_getopt, 0xa00000d7

	.globl		_gets
	.set		_gets, 0xa00000e1

	.globl		_getw
	.set		_getw, 0xa00000e6

	.globl		_ioctl
	.set		_ioctl, 0xa00000eb

	.globl		_isatty
	.set		_isatty, 0xa00000f0

	.globl		_isnand
	.set		_isnand, 0xa00000f5

	.globl		_kill
	.set		_kill, 0xa00000fa

	.globl		_ldexp
	.set		_ldexp, 0xa00000ff

	.globl		_lseek
	.set		_lseek, 0xa0000104

	.globl		_malloc
	.set		_malloc, 0xa0000109

	.globl		_memccpy
	.set		_memccpy, 0xa000010e

	.globl		_memchr
	.set		_memchr, 0xa0000113

	.globl		_memcmp
	.set		_memcmp, 0xa0000118

	.globl		_memcpy
	.set		_memcpy, 0xa000011d

	.globl		_memset
	.set		_memset, 0xa0000122

	.globl		_mktemp
	.set		_mktemp, 0xa0000127

	.globl		_open
	.set		_open, 0xa000012c

	.globl		_printf
	.set		_printf, 0xa0000131

	.globl		_putchar
	.set		_putchar, 0xa0000136

	.globl		_puts
	.set		_puts, 0xa000013b

	.globl		_putw
	.set		_putw, 0xa0000140

	.globl		_read
	.set		_read, 0xa0000145

	.globl		_realloc
	.set		_realloc, 0xa000014a

	.globl		__sbrk
	.set		__sbrk, 0xa000014f

	.globl		_setbuf
	.set		_setbuf, 0xa0000154

	.globl		_sighold
	.set		_sighold, 0xa0000159

	.globl		_sigignore
	.set		_sigignore, 0xa000015e

	.globl		_signal
	.set		_signal, 0xa0000163

	.globl		_sigpause
	.set		_sigpause, 0xa0000168

	.globl		_sigrelse
	.set		_sigrelse, 0xa000016d

	.globl		_sigset
	.set		_sigset, 0xa0000172

	.globl		_sprintf
	.set		_sprintf, 0xa0000177

	.globl		_stat
	.set		_stat, 0xa000017c

	.globl		_strcat
	.set		_strcat, 0xa0000181

	.globl		_strchr
	.set		_strchr, 0xa0000186

	.globl		_strcmp
	.set		_strcmp, 0xa000018b

	.globl		_strcpy
	.set		_strcpy, 0xa0000190

	.globl		_strlen
	.set		_strlen, 0xa0000195

	.globl		_strncat
	.set		_strncat, 0xa000019a

	.globl		_strncmp
	.set		_strncmp, 0xa000019f

	.globl		_strncpy
	.set		_strncpy, 0xa00001a4

	.globl		_strrchr
	.set		_strrchr, 0xa00001a9

	.globl		_time
	.set		_time, 0xa00001ae

	.globl		__old_tolower
	.set		__old_tolower, 0xa00001b3

	.globl		__old_toupper
	.set		__old_toupper, 0xa00001b8

	.globl		_ungetc
	.set		_ungetc, 0xa00001bd

	.globl		_unlink
	.set		_unlink, 0xa00001c2

	.globl		_utime
	.set		_utime, 0xa00001c7

	.globl		_write
	.set		_write, 0xa00001cc

	.globl		_getpid
	.set		_getpid, 0xa00002e9
/*
 * 	Data storage in the shared library
 */
	.globl		___Fp_Used
	.set		___Fp_Used, 0xa0400000

	.globl		__libc__ctype
	.set		__libc__ctype, 0xa0400004

	.globl		__libc_malloc
	.set		__libc_malloc, 0xa0400008

	.globl		__libc_realloc
	.set		__libc_realloc, 0xa040000c

	.globl		__libc_free
	.set		__libc_free, 0xa0400010

	.globl		__libc__allocs
	.set		__libc__allocs, 0xa0400014

	.globl		__libc__sibuf
	.set		__libc__sibuf, 0xa0400018

	.globl		__libc__sobuf
	.set		__libc__sobuf, 0xa040001c

	.globl		__libc__smbuf
	.set		__libc__smbuf, 0xa0400020

	.globl		__libc__iob
	.set		__libc__iob, 0xa0400024

	.globl		__libc__lastbuf
	.set		__libc__lastbuf, 0xa0400028

	.globl		__libc__bufendtab
	.set		__libc__bufendtab, 0xa040002c

	.globl		__libc_end
	.set		__libc_end, 0xa0400030

	.globl		__libc__cleanup
	.set		__libc__cleanup, 0xa0400034

	.globl		__libc_environ
	.set		__libc_environ, 0xa0400038

	.globl		_opterr
	.set		_opterr, 0xa040003c

	.globl		_optind
	.set		_optind, 0xa0400040

	.globl		_optopt
	.set		_optopt, 0xa0400044

	.globl		_optarg
	.set		_optarg, 0xa0400048

	.globl		_errno
	.set		_errno, 0xa040004c

	.globl		___fltused
	.set		___fltused, 0xa0400094
