#define	vector(lbl)	\
	.text;		\
	.globl	_##lbl;	\
	jmp	_##lbl;

#define illvect		  \
	.globl	_illegal; \
	jmp	_illegal;

	vector	(_doprnt)	 	/* a0000000 */
	vector	(_bufsync)		/* a0000005 */
	vector	(_cleanup)		/* a000000f */
	vector	(_cerror)		/* a000000a */
	vector	(_filbuf)		/* a0000014 */
	vector	(_findbuf)		/* a0000019 */
	vector	(_getstream)		/* a000001e _findiop */
	vector	(_flsbuf)		/* a0000023 */
	vector	(_wrtchk)		/* a0000028 */
	vector	(_xflsbuf)		/* a000002d */
	vector	(abs)			/* a0000032 */
	vector	(access)		/* a0000037 */
	vector	(atof)			/* a000003c */
	vector	(atoi)			/* a0000041 */
	vector	(atol)			/* a0000046 */
	vector	(__brk)			/* a000004b */
	vector	(calloc)		/* a0000050 */
	vector	(cfree)			/* a0000055 */
	vector	(chdir)			/* a000005a */
	vector	(chmod)			/* a000005f */
	vector	(close)			/* a0000064 */
	vector	(creat)			/* a0000069 */
	vector	(ecvt)			/* a000006e */
	vector	(fclose)		/* a0000073 */
	vector	(fcntl)			/* a0000078 */
	vector	(fcvt)			/* a000007d */
	vector	(fflush)		/* a0000082 */
	vector	(fgetc)			/* a0000087 */
	vector	(fgets)			/* a000008c */
	vector	(fopen)			/* a0000091 */
	vector	(fprintf)		/* a0000096 */
	vector	(fputc)			/* a000009b */
	vector	(fputs)			/* a00000a0 */
	vector	(fread)			/* a00000a5 */
	vector	(free)			/* a00000aa */
	vector	(freopen)		/* a00000af */
	vector	(frexp)			/* a00000b4 */
	vector	(fseek)			/* a00000b9 */
	vector	(fstat)			/* a00000be */
	vector	(fwrite)		/* a00000c3 */
	vector	(gcvt)			/* a00000c8 */
	vector	(getchar)		/* a00000cd */
	vector	(getenv)		/* a00000d2 */
	vector	(getopt)		/* a00000d7 */
	illvect				/* a00000dc */
	vector	(gets)			/* a00000e1 */
	vector	(getw)			/* a00000e6 */
	vector	(ioctl)			/* a00000eb */
	vector	(isatty)		/* a00000f0 */
	vector	(isnand)		/* a00000f5 */
	vector	(kill)			/* a00000fa */
	vector	(ldexp)			/* a00000ff */
	vector	(lseek)			/* a0000104 */
	vector	(malloc)		/* a0000109 */
	vector	(__ibcs_memccpy)	/* a000010e */
	vector	(__ibcs_memchr)		/* a0000113 */
	vector	(__ibcs_memcmp)		/* a0000118 */
	vector	(__ibcs_memcpy)		/* a000011d */
	vector	(__ibcs_memset)		/* a0000122 */
	vector	(mktemp)		/* a0000127 */
	vector	(open)			/* a000012c */
	vector	(printf)		/* a0000131 */
	vector	(putchar)		/* a0000136 */
	vector	(puts)			/* a000013b */
	vector	(putw)			/* a0000140 */
	vector	(read)			/* a0000145 */
	vector	(realloc)		/* a000014a */
	vector	(__sbrk)		/* a000014f */
	vector	(setbuf)		/* a0000154 */
	vector	(sighold)		/* a0000159 */
	vector	(sigignore)		/* a000015e */
	vector	(signal)		/* a0000163 */
	vector	(sigpause)		/* a0000168 */
	vector	(sigrelse)		/* a000016d */
	vector	(sigset)		/* a0000172 */
	vector	(sprintf)		/* a0000177 */
	vector	(stat)			/* a000017c */
	vector	(strcat)		/* a0000181 */
	vector	(strchr)		/* a0000186 */
	vector	(strcmp)		/* a000018b */
	vector	(__ibcs_strcpy)		/* a0000190 */
	vector	(__ibcs_strlen)		/* a0000195 */
	vector	(strncat)		/* a000019a */
	vector	(strncmp)		/* a000019f */
	vector	(strncpy)		/* a00001a4 */
	vector	(strrchr)		/* a00001a9 */
	vector	(time)			/* a00001ae */
	vector	(_old_tolower)		/* a00001b3 */
	vector	(_old_toupper)		/* a00001b8 */
	vector	(ungetc)		/* a00001bd */
	vector	(unlink)		/* a00001c2 */
	vector	(utime)			/* a00001c7 */
	vector	(write)			/* a00001cc */
	illvect				/* a00001d1 */
	illvect				/* a00001d6 */
	illvect				/* a00001db */
	illvect				/* a00001e0 */
	illvect				/* a00001e5 */
	illvect				/* a00001ea */
	illvect				/* a00001ef */
	illvect				/* a00001f4 */
	illvect				/* a00001f9 */
	illvect				/* a00001fe */
	illvect				/* a0000203 */
	illvect				/* a0000208 */
	illvect				/* a000020d */
	illvect				/* a0000212 */
	illvect				/* a0000217 */
	illvect				/* a000021c */
	illvect				/* a0000221 */
	illvect				/* a0000226 */
	illvect				/* a000022b */
	illvect				/* a0000230 */
	illvect				/* a0000235 */
	illvect				/* a000023a */
	illvect				/* a000023f */
	illvect				/* a0000244 */
	illvect				/* a0000249 */
	illvect				/* a000024e */
	illvect				/* a0000253 */
	illvect				/* a0000258 */
	illvect				/* a000025d */
	illvect				/* a0000262 */
	illvect				/* a0000267 */
	illvect				/* a000026c */
	illvect				/* a0000271 */
	illvect				/* a0000276 */
	illvect				/* a000027b */
	illvect				/* a0000280 */
	illvect				/* a0000285 */
	illvect				/* a000028a */
	illvect				/* a000028f */
	illvect				/* a0000294 */
	illvect				/* a0000299 */
	illvect				/* a000029e */
	illvect				/* a00002a3 */
	illvect				/* a00002a8 */
	illvect				/* a00002ad */
	illvect				/* a00002b2 */
	illvect				/* a00002b7 */
	illvect				/* a00002bc */
	illvect				/* a00002c1 */
	illvect				/* a00002c6 */
	illvect				/* a00002cb */
	illvect				/* a00002d0 */
	illvect				/* a00002d5 */
	illvect				/* a00002da */
	illvect				/* a00002df */
	illvect				/* a00002e4 */
	vector	(getpid)		/* a00002e9 */

/*
 *  Macros for the data section processing
 */


#define	ldata(lbl)		\
	.data;			\
	.globl	_##lbl;		\
_##lbl:	.long	0;

#define	ldatac(lbl,val)		\
	.data;			\
	.globl	_##lbl;		\
_##lbl:	.long	val;

#define	laddr(lbl,val)		\
	.data;			\
	.globl	_##lbl, _##val;	\
_##lbl:	.long	_##val;

#define	filler			\
	.data;			\
	.long	0;

	ldata	(__Fp_Used)		/* a0400000 */
	laddr	(_libc__ctype,_ctype)	/* a0400004 */
	laddr	(_libc_malloc,malloc)	/* a0400008 */
	laddr	(_libc_realloc,realloc)	/* a040000c */
	laddr	(_libc_free,free)	/* a0400010 */
	ldata	(_libc__allocs)		/* a0400014 */
	ldata	(_bufin)		/* a0400018 */
	ldata	(_bufout)		/* a040001c */
	ldata	(_buferr)		/* a0400020 */
	ldata	(_iob)			/* a0400024 */
	ldata	(_lastiob)		/* a0400028 */
	ldata	(_lastbuf)		/* a040002c */
	ldata	(_libc_end)		/* a0400030 */
	ldata	(_libc__cleanup)        /* a0400034 */
	ldata	(_libc_environ)		/* a0400038 */
	ldatac	(opterr,1)		/* a040003c */
	ldatac	(optind,1)		/* a0400040 */
	ldata	(optopt)		/* a0400044 */
	ldata	(optarg)		/* a0400048 */
	ldata	(errno)			/* a040004c */

	filler				/* a0400050 */
	filler				/* a0400054 */
	filler				/* a0400058 */
	filler				/* a040005c */
	filler				/* a0400060 */
	filler				/* a0400064 */
	filler				/* a0400068 */
	filler				/* a040006c */
	filler				/* a0400070 */
	filler				/* a0400074 */
	filler				/* a0400078 */
	filler				/* a040007c */
	filler				/* a0400080 */
	filler				/* a0400084 */
	filler				/* a0400088 */
	filler				/* a040008c */
	filler				/* a0400090 */

	ldata	(__fltused)		/* a0400094 */
